#!/bin/sh

USB_SRC=/media/usb/
USB_DEVICE=/dev/sda1

# Backup / Restore sysfiles
SCRIPTNAME=restore_sysfiles.sh
CFG_FILE=${USB_SRC}sw_update/backup_restore_cfg
SYSFILES_BACKUP_DIR=${USB_SRC}sw_update/bak_sysconfig
# Logging
UPDATE_REPORT_FILE="${USB_SRC}sw_update/update_report.txt"
USB_UPDATE_ROOT_PATH="${USB_SRC}sw_update"
# Variables
REQUIREMENTS_MET=TRUE


echo "-----------------------------------------------------------------"
echo "${SCRIPTNAME}: entering script."

# Test requirements: usb mounted?
if [ ! -e $USB_UPDATE_ROOT_PATH ] ; then
	echo "${SCRIPTNAME}: $USB_UPDATE_ROOT_PATH not found"
	REQUIREMENTS_MET=FALSE
fi

# Test requirements: backupfolder existing?
if [ ! -e $SYSFILES_BACKUP_DIR ] ; then
	echo "${SCRIPTNAME}: backupfolder ${SYSFILES_BACKUP_DIR} not found" >> "$UPDATE_REPORT_FILE"
	echo "${SCRIPTNAME}: backupfolder ${SYSFILES_BACKUP_DIR} not found"
	REQUIREMENTS_MET=FALSE
fi

if [ $REQUIREMENTS_MET = TRUE ] ; then
	# read configuration file
	echo "${SCRIPTNAME}: restoring files."
	while IFS= read -r; do
		FILENAME=$(basename $REPLY)
		SOURCE="${SYSFILES_BACKUP_DIR}/${FILENAME}"
		if [ -d ${SOURCE} ] ; then
			DESTINATION=$(dirname $REPLY)
			echo "copy folder SOURCE=${SOURCE} to DEST=${DESTINATION}"
			# echo "${SOURCE} is a directory"
			cp --verbose --recursive --force ${SOURCE} ${DESTINATION}
			echo "cp resulted ${?}"
			if [ $? != 0 ] ; then
				echo "cp failed, errorcode=${?}"
			else
				echo "cp succeeded"
			fi
		elif [ -f ${SOURCE} ] ; then
			# echo "${SOURCE} is a regular file"
			DESTINATION=${REPLY}
			DEST_FOLDER_STRUCT=$(dirname ${DESTINATION})
			if [ ! -e ${DEST_FOLDER_STRUCT} ] ; then
				# echo "folder structure not found, creating ${DEST_FOLDER_STRUCT}"
				mkdir -p ${DEST_FOLDER_STRUCT}
			fi
			cp --verbose --force ${SOURCE} ${DESTINATION}
		else
			echo "${SCRIPTNAME}: ${SOURCE} not reg. file nor folder." >> "$UPDATE_REPORT_FILE"
			echo "${SCRIPTNAME}: ${SOURCE} not reg. file nor folder."
		fi
	
		# create folder structure if needed
		#FOLDER="$(dirname $REPLY)"
		#echo "Creating folder $FOLDER"
		# mkdir -p $FOLDER
		# cp --force --verbose --recursive $SYSFILES_BACKUP_DIR $REPLY
	done < $CFG_FILE
	sync
else
	echo "${SCRIPTNAME}: done nothing."
fi 

 
          #cp -f "$CONF_BACKUP_DIR"*.conf "$CONF_TARGET_DIR"
          #cp -f "$CONF_BACKUP_DIR".htpasswd "$PASSWD_DIR"
          #chmod a+rw "$CONF_TARGET_DIR"*.conf
          #chmod a+rw "$PASSWD_DIR".htpasswd
          sync
          #"$SET_SYSCONF" "$SYSCONF_DST"

echo "${SCRIPTNAME}: leaving script."
echo "-----------------------------------------------------------------"


        
