#!/bin/sh
# ------------------------------------------------------------------------------
# /etc/script/rootupdate.sh
#
# PMC5XX: updates from USB to eMMC
#
# V1.1,
#
# $1:   1=set bootloader update marker file
#       0=don't set bootloader update marker file
# ------------------------------------------------------------------------------

SET_BOOTLOADER_MARK_FILE="$1"

# ------------------------------------------------------------------------------
# function set_file
# creates an empty file at a specific path on a specific partition
#
# param 1: device/partition
# param 2: path
# ------------------------------------------------------------------------------
function set_file {
  mount "$1" /mnt
  touch "/mnt$2"
  umount /mnt
  return 0
}

BOOTLOADER_MARK_FILE="/mnt/bootloader_update.txt"
SCRIPTNAME=rootupdate.sh
SEPARATOR="============================================================="
echo ${SEPARATOR}
echo "${SCRIPTNAME}: entering script."

# USB
USB_SRC=/media/usb/
USB_DEVICE=/dev/sda1
# 1st root fs
ROOTFS_FILENAME="root.tar.bz2"
ROOTFS_FILEPATH="${USB_SRC}sw_update/rootfs/${ROOTFS_FILENAME}"

ROOTFS_TARGET_PARTITION="/dev/mmcblk1p1" # eMMC, part 1
ROOTFS_TARGET_DEVICE="/dev/mmcblk1" # eMMC


# 2nd root fs
MIN_ROOTFS_FILENAME="min_root_pte.tar.bz2"
MIN_ROOTFS_FILEPATH="${USB_SRC}sw_update/${MIN_ROOTFS_FILENAME}"

ROOTFS_MARK_FILE=/mnt/rootfs_update.txt
UPDATE_REPORT_FILE="$USB_SRC"sw_update/update_report.txt

if [ ! -e $ROOTFS_TARGET_PARTITION ]; then
  echo "$ROOTFS_TARGET_PARTITION doesn't exist - creating it..."
  echo -e "n\np\n1\n\n\nt\n83\nw\n" | fdisk $ROOTFS_TARGET_DEVICE
  sleep 3
  mkfs.ext4 $ROOTFS_TARGET_PARTITION
else
  echo "$ROOTFS_TARGET_PARTITION does exist, all good."
fi

# try to kill all processes...
echo ${SEPARATOR}
echo "${SCRIPTNAME}: Killing processes"

killall startx
killall telnetd
/etc/init.d/pureftpd stop
/etc/init.d/dbus stop
killall csmonitor.sh
killall codesyscontrol
killall crond
killall sshd
killall udevd
killall inetd
killall run-parts
killall sdai_demo
killall sshd
killall telnetd
killall5 -9
sleep 1

echo ${SEPARATOR}
echo "${SCRIPTNAME}: mounting tmpfs."
# create tmpfs folder for pivot_root-environment (2nd root)
rm -rf /new-root
mkdir /new-root
mount -n -t tmpfs -o size=500M none /new-root

# copy update archive to 2nd-root
echo ${SEPARATOR}
echo "${SCRIPTNAME}: copying main rootfs archive."
cp ${ROOTFS_FILEPATH} /new-root
chmod 755 /new-root/${ROOTFS_FILENAME}

# deflate 2nd-root-FS from archive
echo ${SEPARATOR}
echo "${SCRIPTNAME}: extracting min_root, please wait...(${MIN_ROOTFS_FILEPATH})"
cd /new-root
mkdir old-root
tar -xjpf ${MIN_ROOTFS_FILEPATH} -C /new-root
# erase 2nd root archive to save space in tmpfs
rm -rf /new-root/${MIN_ROOTFS_FILENAME}

# create proc/sysfs/dev in 2nd-root
echo ${SEPARATOR}
echo "${SCRIPTNAME}: creating /proc /sys /dev."
cd /new-root
mount -t proc none proc
mount -n -t sysfs none sys
mount -o bind /dev /new-root/dev

# unmount stuff from 1st-root
echo ${SEPARATOR}
echo "${SCRIPTNAME}: unmounting old root fs."
umount -f /proc
umount -f /tmp
umount -f /media
umount -f /sys
umount -f /var/log
umount -f /var/run
umount -f /var/lock
umount -f /var/tmp
umount -f /runl
umount -f /sys/fs/fuse/connections
umount -f /dev/pts
umount -f /sys

# switch root to 2nd-root
echo ${SEPARATOR}
echo "${SCRIPTNAME}: switching ROOT"
pivot_root . old-root
cd /

# mount 1st-root, erase all and deflate archive to 1st-root
echo ${SEPARATOR}
echo "${SCRIPTNAME}: extracting main root fs, please be patient...(${ROOTFS_FILEPATH})"

mount ${ROOTFS_TARGET_PARTITION} /mnt
rm -rf /mnt/*
tar -xjpf ${ROOTFS_FILENAME} -C /mnt
sync

# set marker file in new 1st-root
echo ${SEPARATOR}
echo "${SCRIPTNAME}: setting marker (${ROOTFS_MARK_FILE})"
touch "$ROOTFS_MARK_FILE"
if [ $SET_BOOTLOADER_MARK_FILE -eq 1 ];
then
  echo "${SCRIPTNAME}: setting marker ($BOOTLOADER_MARK_FILE)"
  touch "$BOOTLOADER_MARK_FILE"
  fsync "$BOOTLOADER_MARK_FILE"  
else
  echo "${SCRIPTNAME}: not setting marker ($BOOTLOADER_MARK_FILE)"
fi

chmod a+rw "$ROOTFS_MARK_FILE"
sync
fsync "$ROOTFS_MARK_FILE"
sleep 1

# unmount 1st-root-media
echo ${SEPARATOR}
echo "${SCRIPTNAME}: unmounting flash media."
sync
cd /
umount /mnt
sleep 1
# erase content of /mnt
rm -rf /mnt/*

# unmount all
umount -af
sleep 2

# do a reboot
echo ${SEPARATOR}
echo "${SCRIPTNAME}: doing a reboot..."
echo ${SEPARATOR}
reboot
echo b > /proc/sysrq-trigger                                                              
exit 0
