#!/bin/sh
# /etc/script/update.sh
#
# Main script usb update eMMC PTE5XX/PMC5XX 
#
# V1.2
#

#config file settings
SYSTEM_CONFIG_FILE=/home/user/conf/syspar.conf
SET_SYSTEM=/usr/bin/set_sysconf_pte

#path settings used for files to update
USB_SRC=/media/usb/
USB_DEVICE=/dev/sda1
ROOTFS_TARGET_DEVICE="/dev/mmcblk1p1" # eMMC, part 1

# location of update.ini file
UPDATE_CONFIG_FILE="$USB_SRC"sw_update/update.ini

# update from USB stick
SCRIPTNAME=update.sh

# update scripts (paths from usb stick)
BACKUP_SYSFILES_SH="/media/usb/sw_update/script/backup_sysfiles.sh"
RESTORE_SYSFILES_SH="/media/usb/sw_update/script/restore_sysfiles.sh"
ROOTUPDATE_SH="/media/usb/sw_update/script/rootupdate.sh"
UPDATE_FILES_SH="/media/usb/sw_update/script/update_files.sh"

# update scripts (paths from internal rootfs)
#BACKUP_SYSFILES_SH="/etc/script/backup_sysfiles.sh"
#RESTORE_SYSFILES_SH="/etc/script/restore_sysfiles.sh"
#ROOTUPDATE_SH="/etc/script/rootupdate.sh"
#UPDATE_FILES_SH="/etc/script/update_files.sh"

# kernel
KERNEL_IMG="$USB_SRC"sw_update/kernel/root.tar.bz2
KERNEL_MD5_FILE="$USB_SRC"sw_update/kernel/kernel_md5.chk
# rootfs
ROOTFS_IMG="$USB_SRC"sw_update/rootfs/root.tar.bz2
ROOTFS_MD5_FILE="$USB_SRC"sw_update/rootfs/rootfs_md5.chk
ROOTFS_MARK_FILE=/rootfs_update.txt
ROOTFS_ERR_FILE="$USB_SRC"sw_update/rootfs/rootfs_err_file.txt
# bootloader
BOOTLOADER_MARK_FILE=/bootloader_update.txt
PRELOADER_IMG="$USB_SRC"sw_update/bootloader/preloader-mkpimage.bin
PRELOADER_MD5_FILE="$USB_SRC"sw_update/bootloader/preloader_md5.chk
UBOOT_IMG="$USB_SRC"sw_update/bootloader/u-boot.img
UBOOT_MD5_FILE="$USB_SRC"sw_update/bootloader/u-boot_md5.chk
BLDR_IMG="$USB_SRC"sw_update/bootloader/bootloader.bin
BLDR_MD5_FILE="$USB_SRC"sw_update/bootloader/bootloader_md5.chk

BLDR_TARGET_DEVICE="/dev/mtd0"
BOOTLOADER_APPLIED=0

REQUIREMENTS_BOOTLOADER_MET=TRUE

UPDATE_REPORT_FILE="$USB_SRC"sw_update/update_report.txt

PASSWD_DIR=/home/user/
PASSWD_FILE="$PASSWD_DIR".htpasswd

UPDATE_MSG_DEFAULT="Updating system. Please wait for message 'update finished' \
(device may reboot). Don't turn off power!"


# ------------------------------------------------------------------------------
# function show_message
# param 1: message string
# param 2: color (white/red/green)
# ------------------------------------------------------------------------------
function show_message() {

  FONT_COLOR=1 # 1=white, 0==black
  FONT_SIZE=15

  # kill prev. instance of dfb_message
  pidof dfb_message > /dev/null 2>&1
  if [ $? -eq 0 ]; then
    killall dfb_message
  fi
  sleep 1
  
  # parse color parameter
  if [ $2 = "red" ]; then
    RGB="255 0 0"
    FONT_COLOR=0 # black
  elif [ $2 = "green" ]; then
    RGB="0 255 0"
    FONT_COLOR=0 # black
  elif [ $2 = "white" ] || [ -z $2 ]; then
    RGB="255 255 255"
    FONT_COLOR=0 # black
  else 
    RGB="0 0 0" # black
    FONT_COLOR=1 # white
  fi
  
  dfb_message "$1" $RGB $FONT_SIZE $FONT_COLOR &
}

# ------------------------------------------------------------------------------
# function test_file
# tests a file at a specific path on a specific partition
#
# param 1: device
# param 2: path
# ------------------------------------------------------------------------------
function test_file {
  mount "$1" /mnt
  echo "update: testing $2 on $1"
  if [ -e "/mnt$2" ];
  then
    umount /mnt
    echo "update: found $2 on $1"
    return 0
  else
    umount /mnt
    echo "update: didn't find $2 on $1"
    return 1
  fi
}

# ------------------------------------------------------------------------------
# function set_file
# creates an empty file at a specific path on a specific partition
#
# param 1: device
# param 2: path
# ------------------------------------------------------------------------------
function set_file {
  mount "$1" /mnt
  touch "/mnt$2"
  umount /mnt
  return 0
}

# ------------------------------------------------------------------------------
# function delete_file
# deletes a file at a specific path on a specific partition
#
# param 1: device
# param 2: path
# ------------------------------------------------------------------------------
function delete_file {
  mount "$1" /mnt
  rm -f "/mnt$2"
  umount /mnt
  return 0
}

# ------------------------------------------------------------------------------
# main 
# ------------------------------------------------------------------------------
echo "-----------------------------------------"
echo "${SCRIPTNAME}: entering script."

signalize_and_halt=0
update_error=0

# test if we run on the correct device
# !!! First version. Must be changed
FREESCALE=`grep Freescale /proc/cpuinfo`
if [ $? != 0 ]; then
	echo "Wrong device"
	exit 1
fi
 
# test on tool read_update_config - use from usb stick if available
if [ -f "/media/usb/sw_update/script/read_update_config" ]; then
  RD_CFG_TOOL="/media/usb/sw_update/script/read_update_config"
else
  if [ -e "/usr/bin/read_update_config" ]; then
    RD_CFG_TOOL="/usr/bin/read_update_config"
  else
    touch -c "$UPDATE_REPORT_FILE"
    echo "read_update_config not found."
    echo "read_update_config not found." >> "$UPDATE_REPORT_FILE"
    umount ${USB_DEVICE}
    exit 1  
  fi
fi

echo "using ${RD_CFG_TOOL}"

if [ -e "$UPDATE_CONFIG_FILE" ]; then
  # Create report file
  touch -c "$UPDATE_REPORT_FILE"                                                                   
  echo "-----------------------------------------------------------------" \
  >> "$UPDATE_REPORT_FILE"
  date >> "$UPDATE_REPORT_FILE"  
  echo "found update.ini on usb"
  echo "found update.ini on usb storage" >> "$UPDATE_REPORT_FILE"
  ${RD_CFG_TOOL} "$UPDATE_CONFIG_FILE"
  RetValue="$?"
  
  
  # SIGNALIZATION TODO: Turn off leds
      
  echo "read_update_config resulted: ${RetValue}"
  if [ ${RetValue} -eq 255 ]; then
    # read_update_config=255: error case    
    echo "error in update config file, terminating update"
    echo "error in update config file, terminating update" >> "$UPDATE_REPORT_FILE"
    # do the cleanup and halt
    sync
    /bin/umount /dev/sda1
    # SIGNALIZATION TODO: Turn off leds
    # Signalize the user: LEDs H4,H16,H17 are red steadily in error case
    show_message "Update finished (unsucessful): error in update config file, terminating update." red
    exit 1
  elif [ ${RetValue} -gt 0 ]; then # if [ ${RetValue} -eq 255 ]
    # if we got here (an update is configured on the usb stick)
    # the system will signalize via LEDs and halt in any case, i.e.
    # in case of successful or unsucessful update
    signalize_and_halt=1 
    # read_update_config>0: normal case
    # SIGNALIZATION TODO: Turn off leds
    # Signalize the user: LEDs H4,H16,H17 blink GN slow while doing update
    echo "update.ini successfully read" >> "$UPDATE_REPORT_FILE"
    sleep 1
    
    # ----------------------------------------------------------------
    # UPDATE BOOTLOADER
    # ----------------------------------------------------------------
    #if [ ${RetValue} -ge 8 ]; then
    if test $((${RetValue} & 8)) -eq 8; then
      test_file "$ROOTFS_TARGET_DEVICE" "$BOOTLOADER_MARK_FILE"
      if [ $? -ne 0 ]; # if marker file doesn't exist (ie bootloader not already applied)
      then
        if test -e ${BLDR_IMG} && test -e ${BLDR_MD5_FILE} ; then
          # Found single bootloader file bootloader.bin -> write single bootloader file
          echo "Found ${BLDR_IMG}"
          echo "Found ${BLDR_MD5_FILE}"
          # test checksum and and write bootloader
          MD5_CALC=$(/usr/bin/md5sum "${BLDR_IMG}" | cut -f 1 -d ' ')
          echo "MD5=$MD5_CALC (calculated)"
          MD5_STORED=$(cat "${BLDR_MD5_FILE}")
          echo "MD5=$MD5_STORED ($(basename "${BLDR_MD5_FILE}"))"
          if [ $MD5_CALC != $MD5_STORED ] ; then
            echo "bootloader update: checksum error bootlader (error)"
            echo "bootloader update: checksum error bootloader (error)" \
            >> "$UPDATE_REPORT_FILE"
            update_error=1
          else
            echo "Writing bootloader"
            echo "bootloader update: writing bootloader" \
            >> "$UPDATE_REPORT_FILE"
            show_message "$UPDATE_MSG_DEFAULT" black
            flash_erase ${BLDR_TARGET_DEVICE} 0 256
            sleep 1          
            dd if="${BLDR_IMG}" of=${BLDR_TARGET_DEVICE}
            echo "if=${BLDR_IMG}" # REMOVE
            echo "of=${BLDR_TARGET_DEVICE}" # REMOVE
            BOOTLOADER_APPLIED=1
            sync
            sleep 1
          fi
        else # if test -e ${BLDR_IMG} && test -e ${BLDR_MD5_FILE} ; then
          echo "bootloader update: no bootloader-image and/or checksum file found (error)"
          echo "bootloader update: no bootloader-image and/or checksum file found (error)" \
          >> "$UPDATE_REPORT_FILE"
          REQUIREMENTS_BOOTLOADER_MET=FALSE
          update_error=1
        fi # if test -e ${BLDR_IMG} && test -e ${BLDR_MD5_FILE} ; then
        RetValue=$(( $RetValue-8 ))
        echo ${RetValue}
      else #  # if [ $? -ne 0 ];
        # we're through - remove marker file again
        delete_file "$ROOTFS_TARGET_DEVICE" "$BOOTLOADER_MARK_FILE"      
      fi # if [ $? -ne 0 ];
    fi # if [ ${RetValue} -ge 8 ]; then         
    
    # ----------------------------------------------------------------
    # ROOTFS UPDATE
    # ----------------------------------------------------------------
    # if [ ${RetValue} -ge 64 ]; then
    if test $((${RetValue} & 64)) -eq 64; then
      # read_update_config&=64: rootfs update
      
      test_file "$ROOTFS_TARGET_DEVICE" "$ROOTFS_MARK_FILE"
      if [ $? -eq 0 ]; then
        # rootfs update step 2: restore backup files
        echo "rootfs update: restoring backup of systemfiles"
        echo "rootfs update: restoring backup of systemfiles" >> "$UPDATE_REPORT_FILE"
        show_message "$UPDATE_MSG_DEFAULT" black
        
        rm "$ROOTFS_MARK_FILE"
        ${RESTORE_SYSFILES_SH}
        #/etc/script/restore_sysfiles.sh 
      else # if [ -e "$ROOTFS_MARK_FILE" ]; then
        # rootfs update step 1: backup system files
        if [ -e "$ROOTFS_MD5_FILE" ]; then
          if [ -e "$ROOTFS_IMG" ]; then
            show_message "$UPDATE_MSG_DEFAULT" black
            echo "rootfs update: message begin rootfs update"
            echo "rootfs update: message begin rootfs update" >> "$UPDATE_REPORT_FILE"
            if [ -e "$ROOTFS_ERR_FILE" ]; then
              rm "$ROOTFS_ERR_FILE"
            fi
            MD5_1=$(/usr/bin/md5sum "$ROOTFS_IMG"  | cut  -f 1 -d ' ')
            #/usr/bin/md5sum ubi.img  | cut  -f 1 -d ' ' > rootfs_md5.chk
            echo "$MD5_1: checksum of rootfs archive"
            MD5_2=$(cat "$ROOTFS_MD5_FILE")
            echo "$MD5_2: checksum stored"
            if [ $MD5_1 != $MD5_2 ];then
              echo "rootfs update: error checksum image file"
              echo "rootfs update: error checksum image file" >> "$UPDATE_REPORT_FILE"
              update_error=1
              sleep 1
            else
              # save a backup of system files
              echo "rootfs update: storing backup of sysfiles."
              echo "rootfs update: storing backup of sysfiles." >> "$UPDATE_REPORT_FILE"
              ${BACKUP_SYSFILES_SH}
              # /etc/script/backup_sysfiles.sh
              # update rootfs
              "$ROOTUPDATE_SH" "$BOOTLOADER_APPLIED"
              #. /etc/script/rootupdate.sh
            fi
          else # if [ -e "$ROOTFS_IMG" ]; then
            echo "rootfs update: rootfs image file not found"
            echo "rootfs update: rootfs image file not found" >> "$UPDATE_REPORT_FILE"
            update_error=1
            sleep 1
          fi # if [ -e "$ROOTFS_IMG" ]; then
        else # if [ -e "$ROOTFS_MD5_FILE" ]; then
          echo "rootfs update: error rootfs checksum file not found"
          echo "rootfs update: error rootfs checksum file not found" >> "$UPDATE_REPORT_FILE"
          update_error=1
          sleep 1
        fi # if [ -e "$ROOTFS_MD5_FILE" ]; then
      fi # if [ -e "$ROOTFS_MARK_FILE" ]; then
      RetValue=$(( $RetValue-64 ))
      echo ${RetValue}
    fi # if [ ${RetValue} -ge 64 ]; then
    
    # ----------------------------------------------------------------
    # KERNEL UPDATE
    # ----------------------------------------------------------------
    #if [ ${RetValue} -ge 32 ]; then
    if test $((${RetValue} & 32)) -eq 32; then
      if [ -e "$KERNEL_IMG" ]; then
	if [ -e "$KERNEL_MD5_FILE" ]; then
	  echo "kernel update: message begin kernel update" >> "$UPDATE_REPORT_FILE"
	  MD5_1=$(/usr/bin/md5sum "$KERNEL_IMG"  | cut  -f 1 -d ' ')
	  #/usr/bin/md5sum uImage  | cut  -f 1 -d ' ' > kernel_md5.chk
	  echo $MD5_1
	  MD5_2=$(cat "$KERNEL_MD5_FILE")
	  echo $MD5_2
	  if [ $MD5_1 != $MD5_2 ];then
	    echo "kernel update: error checksum kernel image"
	    echo "kernel update: error checksum kernel image" >> "$UPDATE_REPORT_FILE"
	    update_error=1
	    sleep 1
	  else # [ $MD5_1 != $MD5_2 ];then
	    echo "kernel update: message checksum ok, write image"
	    echo "kernel update: message checksum ok, write image" >> "$UPDATE_REPORT_FILE"
	    tar xjvf "$KERNEL_IMG"  -C /boot
	    if [ $? -eq 0 ] ; then
	      echo "kernel update: message write image ok"
	      echo "kernel update: message write image ok" >> "$UPDATE_REPORT_FILE"
	    else
	      echo "kernel update: error write image"
	      echo "kernel update: error write image" >> "$UPDATE_REPORT_FILE"
	      update_error=1
	      sleep 1
	    fi
	  fi # [ $MD5_1 != $MD5_2 ];then
	else # if [ -e "$KERNEL_MD5_FILE" ]; then
	  echo "kernel update: error checksum file not found"
	  echo "kernel update: error checksum file not found" >> "$UPDATE_REPORT_FILE"
	  update_error=1
	  sleep 1
	fi # if [ -e "$KERNEL_MD5_FILE" ]; then
      else # if [ -e "$KERNEL_IMG" ]; then
	echo "kernel update: error kernel image file not found"
	echo "kernel update: error kernel image file not found" >> "$UPDATE_REPORT_FILE"
	update_error=1
	sleep 1
      fi # if [ -e "$KERNEL_IMG" ]; then
      RetValue=$(( $RetValue-32 ))
      echo ${RetValue}
    fi # if [ ${RetValue} -ge 32 ]; then
    
    # ----------------------------------------------------------------
    # UPDATE FILES
    # ----------------------------------------------------------------
    # if [ ${RetValue} -ge 1 ];
    if test $((${RetValue} & 1)) -eq 1;
    then
      ${UPDATE_FILES_SH}
      # /etc/script/update_files.sh
      RES=$?
      if [ $RES != 0 ];
      then
	echo "update_files.sh failed (result=$RES)"
	update_error=1
      fi
      RetValue=$(( $RetValue - 1 ))
    fi # if [ ${RetValue} -ge 1 ] 			

    # ----------------------------------------------------------------
    # Signalization LEDs and HALT        
    # ----------------------------------------------------------------
    if  [ $signalize_and_halt -eq 1 ] ; then
      if [ $update_error -eq 0 ] ; then
        echo "update successful, remove usb-stick and reboot panel"
        echo "update successful" >> "$UPDATE_REPORT_FILE"
        # do the cleanup and halt
        sync
        /bin/umount /dev/sda1
        # SIGNALIZATION TODO: Turn off leds
        # Signalize the user: LEDs H4,H16,H17 are green steadily in ok case
        show_message "Update finished (successful). Please remove usb-stick and reboot panel." green        
        rm -f /etc/group+
        rm -f /etc/group-
        sync
        sleep 10
        exit 0
      else # if [ $update_error -eq 0 ]
        echo "update terminated with error, please remove usb-stick"
        echo "update terminated with error, please remove usb-stick" \
                              >> "$UPDATE_REPORT_FILE"
        # do the cleanup and halt
        sync
        /bin/umount /dev/sda1
        # SIGNALIZATION TODO: Turn off leds
        # Signalize the user: LEDs H4,H16,H17 are red steadily in error case
        show_message "Update finished (unsucessful). Please remove usb-stick." red
        exit 1
      fi # if [ $update_error -eq 0 ]
    fi # if  [ $signalize_and_halt -eq 1 ]
  elif [ ${RetValue} -eq 0 ]; then # if [ ${RetValue} -eq 255 ]
    echo "nothing to update"
    echo "nothing to update" >> "$UPDATE_REPORT_FILE"
    sleep 1
  else # # if [ ${RetValue} -eq 255 ]
    echo "read_update_config terminated with error"
    echo "read_update_config terminated with error" >> "$UPDATE_REPORT_FILE"
    sleep 1
  fi # # if [ ${RetValue} -eq 255 ]
else #  if [ -e "$UPDATE_CONFIG_FILE" ]
  echo "${SCRIPTNAME}: no update config file not found."
fi #  if [ -e "$UPDATE_CONFIG_FILE" ]

echo "${SCRIPTNAME}: leaving script."
echo "----------------------------------------"

