#!/bin/sh
#
# Do update files from usb stick to rootfs
# 

USB_SRC=/media/usb/
USB_DEVICE=/dev/sda1

SCRIPTNAME=update_files.sh
SOURCE_PATH_BASE="${USB_SRC}sw_update/fileupdate"
MOUNT_PATH="/mnt_flash"

#FLASH_DEVICE="/dev/mmcblk0p1" # sd card intern, part 1
FLASH_DEVICE="/dev/mmcblk1p1" # eMMC, part 1

CFG_FILE="${USB_SRC}sw_update/fileupdate_cfg"
REQUIREMENTS_FILEUPDATE_MET=TRUE
UPDATE_REPORT_FILE=${USB_SRC}sw_update/update_report.txt
REQUIREMENTS_MET=TRUE
update_error=0
RES=0
SEPARATOR="============================================================="

echo ${SEPARATOR}
echo "${SCRIPTNAME}: entering script"

# Check if source folder exists
if [ ! -d "$SOURCE_PATH_BASE" ]; then 
	echo "ERROR: file update: source folder not found (error)"
	echo "ERROR: file update: source folder not found (error)" \
		>> "$UPDATE_REPORT_FILE"
	REQUIREMENTS_FILEUPDATE_MET=FALSE
fi

# Mount rootfs partition
if [ ! -d "$MOUNT_PATH" ];
then
	mkdir "$MOUNT_PATH"
fi
mount "$FLASH_DEVICE" "$MOUNT_PATH"
RES=$?
if [ ! $RES -eq 0 ];
then
	echo "ERROR: file update: failed mounting ${FLASH_DEVICE} (error=$RES)"
	echo "ERROR: file update: failed mounting ${FLASH_DEVICE} (error=$RES)" \
		>> "$UPDATE_REPORT_FILE"
	REQUIREMENTS_FILEUPDATE_MET=FALSE
else
	echo "file update: mounted ${FLASH_DEVICE} (result=$RES)"
fi

# Check configuration file
if [ ! -e "$CFG_FILE" ];
then
	echo "ERROR: file update: configuration file ${CFG_FILE} not found"
	echo "ERROR: file update: configuration file ${CFG_FILE} not found" \
		>> "$UPDATE_REPORT_FILE"
	REQUIREMENTS_FILEUPDATE_MET=FALSE
fi 


if [ $REQUIREMENTS_FILEUPDATE_MET = FALSE ];
then
	update_error=1
else
	echo "file update: copying files..."
	echo "file update: copying files..." >> "$UPDATE_REPORT_FILE"

	while IFS= read -r; do
	    # skip empty lines
		if [ -z ${REPLY} ]; then
		  echo "${REPLY} is empty, skipping."
		  continue
		fi	
		FILENAME=$(basename $REPLY)
		SOURCE="${SOURCE_PATH_BASE}/${REPLY}"

		if [ -d ${SOURCE} ];
		then
			# echo "${SOURCE} is a directory"	# REMOVE
			DESTINATION="${MOUNT_PATH}/$(dirname $REPLY)"
			echo "copy folder SOURCE=${SOURCE} to DEST=${DESTINATION}"
			cp --verbose --recursive --force ${SOURCE} ${DESTINATION}
			RES=$?
			# echo "cp(file) resulted $RES"	# REMOVE
			if [ $RES != 0 ];
			then
				echo "ERROR: file update: cp failed, SRC=${SOURCE}, DST=${DESTINATION}, error=$RES"
        echo "ERROR: file update: cp failed, SRC=${SOURCE}, DST=${DESTINATION}, error=$RES" \
					>> "$UPDATE_REPORT_FILE"
				update_error=1
			fi
		elif [ -f ${SOURCE} ]; # if [ -d ${SOURCE} ]
		then
			# echo "${SOURCE} is a regular file" 	# REMOVE
			DESTINATION="${MOUNT_PATH}/${REPLY}"
			DEST_FOLDER_STRUCT="$(dirname ${DESTINATION})"
			if [ ! -e ${DEST_FOLDER_STRUCT} ];
			then
				# echo "folder structure not found, creating ${DEST_FOLDER_STRUCT}" # REMOVE
				mkdir -p ${DEST_FOLDER_STRUCT}
			# else	
				# echo "folder structure \"${DEST_FOLDER_STRUCT}\" found" # REMOVE
			fi
			cp --verbose --force ${SOURCE} ${DESTINATION}
			RES=$?
			# echo "cp(dir) resulted $RES"	# REMOVE
			if [ $RES != 0 ];
			then
				echo "ERROR: cp(dir) failed, SRC=${SOURCE}, DST=${DESTINATION}, error=$RES"
        echo "ERROR: cp(dir) failed, SRC=${SOURCE}, DST=${DESTINATION}, error=$RES" \
					>> "$UPDATE_REPORT_FILE"
				update_error=1
			fi
		else # if [ -d ${SOURCE} ]
				echo "ERROR: file update: configured source item ${SOURCE} not found"
        echo "ERROR: file update: configured source item ${SOURCE} not found" \
					>> "$UPDATE_REPORT_FILE"
				update_error=1
		fi # if [ -d ${SOURCE} ]
	done < $CFG_FILE
	sync
	sleep 1
	umount "$MOUNT_PATH"
fi 

echo "${SCRIPTNAME}: leaving script"

if [ $update_error -eq 0 ]; 
then
	exit 0
else
	exit 1
fi



        
